using MediatR;
using Sledgemapper.Api.Infrastructure.Data;
using Sledgemapper.Shared.Entities;
using Sledgemapper.Api.Commands;
using System.Linq;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using Sledgemapper.Api.Notifications;

namespace Sledgemapper.Api.Handlers
{
    public class NewRoomCommandHandler : IRequestHandler<NewRoomCommand, bool>
    {
        private readonly SledgemapperDbContext _dbcontext;

        private readonly IMediator _mediator;

        public NewRoomCommandHandler(IMediator mediator, SledgemapperDbContext dbcontext) { _dbcontext = dbcontext; _mediator = mediator; }

        public async Task<bool> Handle(NewRoomCommand notification, CancellationToken cancellationToken)
        {
            var jsonString = JsonSerializer.Serialize<Room>(notification.Room);
            var session = _dbcontext.Sessions.First(m => m.SessionName == notification.SessionName);
            _dbcontext.MapLogs.Add(new Sledgemapper.Api.Models.MapLog
            {
                Operation = "N",
                SessionId = session.SessionId,
                Type = "R",
                Timestamp = notification.Timestamp,
                Object = jsonString,
                UserId = notification.UserId,
            });
            await _dbcontext.SaveChangesAsync();
            await _mediator.Publish(new NewRoomNotification(session, notification.Room, notification.UserId));
            return true;
        }
    }
}
